# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::BatchAI::Mgmt::V2018_05_01
  #
  # The Azure BatchAI Management API.
  #
  class Jobs
    include MsRestAzure

    #
    # Creates and initializes a new instance of the Jobs class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [BatchAIManagementClient] reference to the BatchAIManagementClient
    attr_reader :client

    #
    # Gets a list of Jobs within the specified Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param jobs_list_by_experiment_options [JobsListByExperimentOptions]
    # Additional parameters for the operation
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<Job>] operation results.
    #
    def list_by_experiment(resource_group_name, workspace_name, experiment_name, jobs_list_by_experiment_options:nil, custom_headers:nil)
      first_page = list_by_experiment_as_lazy(resource_group_name, workspace_name, experiment_name, jobs_list_by_experiment_options:jobs_list_by_experiment_options, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Gets a list of Jobs within the specified Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param jobs_list_by_experiment_options [JobsListByExperimentOptions]
    # Additional parameters for the operation
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_experiment_with_http_info(resource_group_name, workspace_name, experiment_name, jobs_list_by_experiment_options:nil, custom_headers:nil)
      list_by_experiment_async(resource_group_name, workspace_name, experiment_name, jobs_list_by_experiment_options:jobs_list_by_experiment_options, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of Jobs within the specified Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param jobs_list_by_experiment_options [JobsListByExperimentOptions]
    # Additional parameters for the operation
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_experiment_async(resource_group_name, workspace_name, experiment_name, jobs_list_by_experiment_options:nil, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._]+$$')).nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MaxLength': '64'" if !workspace_name.nil? && workspace_name.length > 64
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MinLength': '1'" if !workspace_name.nil? && workspace_name.length < 1
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !workspace_name.nil? && workspace_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'experiment_name is nil' if experiment_name.nil?
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MaxLength': '64'" if !experiment_name.nil? && experiment_name.length > 64
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MinLength': '1'" if !experiment_name.nil? && experiment_name.length < 1
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !experiment_name.nil? && experiment_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?

      max_results = nil
      unless jobs_list_by_experiment_options.nil?
        max_results = jobs_list_by_experiment_options.maxResults
      end

      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.BatchAI/workspaces/{workspaceName}/experiments/{experimentName}/jobs'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'experimentName' => experiment_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version,'maxresults' => max_results},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::JobListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Creates a Job in the given Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param parameters [JobCreateParameters] The parameters to provide for job
    # creation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Job] operation results.
    #
    def create(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:nil)
      response = create_async(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param parameters [JobCreateParameters] The parameters to provide for job
    # creation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def create_async(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:nil)
      # Send request
      promise = begin_create_async(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::Job.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Deletes a Job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def delete(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      response = delete_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      # Send request
      promise = begin_delete_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Gets information about a Job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Job] operation results.
    #
    def get(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      response = get_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets information about a Job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      get_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
    end

    #
    # Gets information about a Job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._]+$$')).nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MaxLength': '64'" if !workspace_name.nil? && workspace_name.length > 64
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MinLength': '1'" if !workspace_name.nil? && workspace_name.length < 1
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !workspace_name.nil? && workspace_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'experiment_name is nil' if experiment_name.nil?
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MaxLength': '64'" if !experiment_name.nil? && experiment_name.length > 64
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MinLength': '1'" if !experiment_name.nil? && experiment_name.length < 1
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !experiment_name.nil? && experiment_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'job_name is nil' if job_name.nil?
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MaxLength': '64'" if !job_name.nil? && job_name.length > 64
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MinLength': '1'" if !job_name.nil? && job_name.length < 1
      fail ArgumentError, "'job_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !job_name.nil? && job_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.BatchAI/workspaces/{workspaceName}/experiments/{experimentName}/jobs/{jobName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'experimentName' => experiment_name,'jobName' => job_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::Job.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List all directories and files inside the given directory of the Job's output
    # directory (if the output directory is on Azure File Share or Azure Storage
    # Container).
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param jobs_list_output_files_options [JobsListOutputFilesOptions] Additional
    # parameters for the operation
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<File>] operation results.
    #
    def list_output_files(resource_group_name, workspace_name, experiment_name, job_name, jobs_list_output_files_options, custom_headers:nil)
      first_page = list_output_files_as_lazy(resource_group_name, workspace_name, experiment_name, job_name, jobs_list_output_files_options, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # List all directories and files inside the given directory of the Job's output
    # directory (if the output directory is on Azure File Share or Azure Storage
    # Container).
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param jobs_list_output_files_options [JobsListOutputFilesOptions] Additional
    # parameters for the operation
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_output_files_with_http_info(resource_group_name, workspace_name, experiment_name, job_name, jobs_list_output_files_options, custom_headers:nil)
      list_output_files_async(resource_group_name, workspace_name, experiment_name, job_name, jobs_list_output_files_options, custom_headers:custom_headers).value!
    end

    #
    # List all directories and files inside the given directory of the Job's output
    # directory (if the output directory is on Azure File Share or Azure Storage
    # Container).
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param jobs_list_output_files_options [JobsListOutputFilesOptions] Additional
    # parameters for the operation
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_output_files_async(resource_group_name, workspace_name, experiment_name, job_name, jobs_list_output_files_options, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._]+$$')).nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MaxLength': '64'" if !workspace_name.nil? && workspace_name.length > 64
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MinLength': '1'" if !workspace_name.nil? && workspace_name.length < 1
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !workspace_name.nil? && workspace_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'experiment_name is nil' if experiment_name.nil?
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MaxLength': '64'" if !experiment_name.nil? && experiment_name.length > 64
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MinLength': '1'" if !experiment_name.nil? && experiment_name.length < 1
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !experiment_name.nil? && experiment_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'job_name is nil' if job_name.nil?
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MaxLength': '64'" if !job_name.nil? && job_name.length > 64
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MinLength': '1'" if !job_name.nil? && job_name.length < 1
      fail ArgumentError, "'job_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !job_name.nil? && job_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'jobs_list_output_files_options is nil' if jobs_list_output_files_options.nil?

      outputdirectoryid = nil
      directory = nil
      linkexpiryinminutes = nil
      max_results = nil
      unless jobs_list_output_files_options.nil?
        outputdirectoryid = jobs_list_output_files_options.outputdirectoryid
      end
      unless jobs_list_output_files_options.nil?
        directory = jobs_list_output_files_options.directory
      end
      unless jobs_list_output_files_options.nil?
        linkexpiryinminutes = jobs_list_output_files_options.linkexpiryinminutes
      end
      unless jobs_list_output_files_options.nil?
        max_results = jobs_list_output_files_options.maxResults
      end

      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.BatchAI/workspaces/{workspaceName}/experiments/{experimentName}/jobs/{jobName}/listOutputFiles'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'experimentName' => experiment_name,'jobName' => job_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version,'outputdirectoryid' => outputdirectoryid,'directory' => directory,'linkexpiryinminutes' => linkexpiryinminutes,'maxresults' => max_results},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::FileListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a list of currently existing nodes which were used for the Job
    # execution. The returned information contains the node ID, its public IP and
    # SSH port.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<RemoteLoginInformation>] operation results.
    #
    def list_remote_login_information(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      first_page = list_remote_login_information_as_lazy(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Gets a list of currently existing nodes which were used for the Job
    # execution. The returned information contains the node ID, its public IP and
    # SSH port.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_remote_login_information_with_http_info(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      list_remote_login_information_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of currently existing nodes which were used for the Job
    # execution. The returned information contains the node ID, its public IP and
    # SSH port.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_remote_login_information_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._]+$$')).nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MaxLength': '64'" if !workspace_name.nil? && workspace_name.length > 64
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MinLength': '1'" if !workspace_name.nil? && workspace_name.length < 1
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !workspace_name.nil? && workspace_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'experiment_name is nil' if experiment_name.nil?
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MaxLength': '64'" if !experiment_name.nil? && experiment_name.length > 64
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MinLength': '1'" if !experiment_name.nil? && experiment_name.length < 1
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !experiment_name.nil? && experiment_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'job_name is nil' if job_name.nil?
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MaxLength': '64'" if !job_name.nil? && job_name.length > 64
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MinLength': '1'" if !job_name.nil? && job_name.length < 1
      fail ArgumentError, "'job_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !job_name.nil? && job_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.BatchAI/workspaces/{workspaceName}/experiments/{experimentName}/jobs/{jobName}/listRemoteLoginInformation'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'experimentName' => experiment_name,'jobName' => job_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::RemoteLoginInformationListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Terminates a job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    def terminate(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      response = terminate_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def terminate_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      # Send request
      promise = begin_terminate_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Creates a Job in the given Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param parameters [JobCreateParameters] The parameters to provide for job
    # creation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Job] operation results.
    #
    def begin_create(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:nil)
      response = begin_create_async(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Creates a Job in the given Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param parameters [JobCreateParameters] The parameters to provide for job
    # creation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_create_with_http_info(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:nil)
      begin_create_async(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:custom_headers).value!
    end

    #
    # Creates a Job in the given Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param parameters [JobCreateParameters] The parameters to provide for job
    # creation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_create_async(resource_group_name, workspace_name, experiment_name, job_name, parameters, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._]+$$')).nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MaxLength': '64'" if !workspace_name.nil? && workspace_name.length > 64
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MinLength': '1'" if !workspace_name.nil? && workspace_name.length < 1
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !workspace_name.nil? && workspace_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'experiment_name is nil' if experiment_name.nil?
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MaxLength': '64'" if !experiment_name.nil? && experiment_name.length > 64
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MinLength': '1'" if !experiment_name.nil? && experiment_name.length < 1
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !experiment_name.nil? && experiment_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'job_name is nil' if job_name.nil?
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MaxLength': '64'" if !job_name.nil? && job_name.length > 64
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MinLength': '1'" if !job_name.nil? && job_name.length < 1
      fail ArgumentError, "'job_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !job_name.nil? && job_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'parameters is nil' if parameters.nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::JobCreateParameters.mapper()
      request_content = @client.serialize(request_mapper,  parameters)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.BatchAI/workspaces/{workspaceName}/experiments/{experimentName}/jobs/{jobName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'experimentName' => experiment_name,'jobName' => job_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::Job.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Deletes a Job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_delete(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      response = begin_delete_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Deletes a Job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_with_http_info(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      begin_delete_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
    end

    #
    # Deletes a Job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._]+$$')).nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MaxLength': '64'" if !workspace_name.nil? && workspace_name.length > 64
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MinLength': '1'" if !workspace_name.nil? && workspace_name.length < 1
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !workspace_name.nil? && workspace_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'experiment_name is nil' if experiment_name.nil?
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MaxLength': '64'" if !experiment_name.nil? && experiment_name.length > 64
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MinLength': '1'" if !experiment_name.nil? && experiment_name.length < 1
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !experiment_name.nil? && experiment_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'job_name is nil' if job_name.nil?
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MaxLength': '64'" if !job_name.nil? && job_name.length > 64
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MinLength': '1'" if !job_name.nil? && job_name.length < 1
      fail ArgumentError, "'job_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !job_name.nil? && job_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.BatchAI/workspaces/{workspaceName}/experiments/{experimentName}/jobs/{jobName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'experimentName' => experiment_name,'jobName' => job_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202 || status_code == 204
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Terminates a job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    #
    def begin_terminate(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      response = begin_terminate_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
      nil
    end

    #
    # Terminates a job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_terminate_with_http_info(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      begin_terminate_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
    end

    #
    # Terminates a job.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_terminate_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'Pattern': '^[-\w\._]+$'" if !resource_group_name.nil? && resource_group_name.match(Regexp.new('^^[-\w\._]+$$')).nil?
      fail ArgumentError, 'workspace_name is nil' if workspace_name.nil?
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MaxLength': '64'" if !workspace_name.nil? && workspace_name.length > 64
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'MinLength': '1'" if !workspace_name.nil? && workspace_name.length < 1
      fail ArgumentError, "'workspace_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !workspace_name.nil? && workspace_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'experiment_name is nil' if experiment_name.nil?
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MaxLength': '64'" if !experiment_name.nil? && experiment_name.length > 64
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'MinLength': '1'" if !experiment_name.nil? && experiment_name.length < 1
      fail ArgumentError, "'experiment_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !experiment_name.nil? && experiment_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, 'job_name is nil' if job_name.nil?
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MaxLength': '64'" if !job_name.nil? && job_name.length > 64
      fail ArgumentError, "'job_name' should satisfy the constraint - 'MinLength': '1'" if !job_name.nil? && job_name.length < 1
      fail ArgumentError, "'job_name' should satisfy the constraint - 'Pattern': '^[-\w_]+$'" if !job_name.nil? && job_name.match(Regexp.new('^^[-\w_]+$$')).nil?
      fail ArgumentError, '@client.api_version is nil' if @client.api_version.nil?
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.BatchAI/workspaces/{workspaceName}/experiments/{experimentName}/jobs/{jobName}/terminate'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'resourceGroupName' => resource_group_name,'workspaceName' => workspace_name,'experimentName' => experiment_name,'jobName' => job_name,'subscriptionId' => @client.subscription_id},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?

        result
      end

      promise.execute
    end

    #
    # Gets a list of Jobs within the specified Experiment.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [JobListResult] operation results.
    #
    def list_by_experiment_next(next_page_link, custom_headers:nil)
      response = list_by_experiment_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a list of Jobs within the specified Experiment.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_by_experiment_next_with_http_info(next_page_link, custom_headers:nil)
      list_by_experiment_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of Jobs within the specified Experiment.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_by_experiment_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::JobListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # List all directories and files inside the given directory of the Job's output
    # directory (if the output directory is on Azure File Share or Azure Storage
    # Container).
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [FileListResult] operation results.
    #
    def list_output_files_next(next_page_link, custom_headers:nil)
      response = list_output_files_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # List all directories and files inside the given directory of the Job's output
    # directory (if the output directory is on Azure File Share or Azure Storage
    # Container).
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_output_files_next_with_http_info(next_page_link, custom_headers:nil)
      list_output_files_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # List all directories and files inside the given directory of the Job's output
    # directory (if the output directory is on Azure File Share or Azure Storage
    # Container).
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_output_files_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::FileListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a list of currently existing nodes which were used for the Job
    # execution. The returned information contains the node ID, its public IP and
    # SSH port.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RemoteLoginInformationListResult] operation results.
    #
    def list_remote_login_information_next(next_page_link, custom_headers:nil)
      response = list_remote_login_information_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Gets a list of currently existing nodes which were used for the Job
    # execution. The returned information contains the node ID, its public IP and
    # SSH port.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_remote_login_information_next_with_http_info(next_page_link, custom_headers:nil)
      list_remote_login_information_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Gets a list of currently existing nodes which were used for the Job
    # execution. The returned information contains the node ID, its public IP and
    # SSH port.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_remote_login_information_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:post, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRestAzure::AzureOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::BatchAI::Mgmt::V2018_05_01::Models::RemoteLoginInformationListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Gets a list of Jobs within the specified Experiment.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param jobs_list_by_experiment_options [JobsListByExperimentOptions]
    # Additional parameters for the operation
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [JobListResult] which provide lazy access to pages of the response.
    #
    def list_by_experiment_as_lazy(resource_group_name, workspace_name, experiment_name, jobs_list_by_experiment_options:nil, custom_headers:nil)
      response = list_by_experiment_async(resource_group_name, workspace_name, experiment_name, jobs_list_by_experiment_options:jobs_list_by_experiment_options, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_by_experiment_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # List all directories and files inside the given directory of the Job's output
    # directory (if the output directory is on Azure File Share or Azure Storage
    # Container).
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param jobs_list_output_files_options [JobsListOutputFilesOptions] Additional
    # parameters for the operation
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [FileListResult] which provide lazy access to pages of the response.
    #
    def list_output_files_as_lazy(resource_group_name, workspace_name, experiment_name, job_name, jobs_list_output_files_options, custom_headers:nil)
      response = list_output_files_async(resource_group_name, workspace_name, experiment_name, job_name, jobs_list_output_files_options, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_output_files_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

    #
    # Gets a list of currently existing nodes which were used for the Job
    # execution. The returned information contains the node ID, its public IP and
    # SSH port.
    #
    # @param resource_group_name [String] Name of the resource group to which the
    # resource belongs.
    # @param workspace_name [String] The name of the workspace. Workspace names can
    # only contain a combination of alphanumeric characters along with dash (-) and
    # underscore (_). The name must be from 1 through 64 characters long.
    # @param experiment_name [String] The name of the experiment. Experiment names
    # can only contain a combination of alphanumeric characters along with dash (-)
    # and underscore (_). The name must be from 1 through 64 characters long.
    # @param job_name [String] The name of the job within the specified resource
    # group. Job names can only contain a combination of alphanumeric characters
    # along with dash (-) and underscore (_). The name must be from 1 through 64
    # characters long.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [RemoteLoginInformationListResult] which provide lazy access to pages
    # of the response.
    #
    def list_remote_login_information_as_lazy(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:nil)
      response = list_remote_login_information_async(resource_group_name, workspace_name, experiment_name, job_name, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_remote_login_information_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
